require 'tablets/data/query'

module Tablets
  # Responsible to fetch and prepare data for jquery-datatables
  class Data
    # Initializes data with tablet and params
    def initialize(tablet, params, controller)
      @tablet = tablet
      @params = params
      @controller = controller
    end

    # Prepares data to render as json
    def as_json(_options = {})
      {
        draw: params[:draw].to_i,
        recordsTotal: query.total,
        recordsFiltered: query.filtered,
        data: data,
        payload: tablet.payload
      }
    end

    private

    attr_reader :params, :tablet, :controller

    # Initializes query with concrete relation
    def query
      @query ||= Tablets::Data::Query.new(concrete_relation,
                                          params,
                                          tablet.columns)
    end

    # Relation with concretized parameters
    def concrete_relation
      tablet.relation(params: relation_params, controller: controller)
    end

    # Relation params. Empty hash by default
    def relation_params
      params[:params] || {}
    end

    # Fetching records and applies process tablet callback on it
    def records
      @records ||= tablet.process(query.fetch)
    end

    # Fetching columns data for each row using column[:data] value:
    # * If column[:data] is symbol send it to the record
    # * If column[:data] is proc calls it on the record
    # Also appends details
    def data
      records.map do |record|
        data = tablet.columns.map.with_index do |column, index|
          cell_data = cell(record, column)
          cell_data = CGI.escapeHTML(cell_data.to_s) unless column[:html_safe]

          [index, cell_data]
        end.to_h

        details = tablet.details(record)
        data[:details] = details unless details.nil?

        data
      end
    end

    # Returns single cell value, for specified record and for specified column
    def cell(record, column)
      case column[:data]
      when Symbol then record.send(column[:data])
      when Proc then column[:data].call(record, @controller)
      end
    end
  end
end
