require 'active_support/core_ext/object/blank'

require 'tablets/engine'

module Tablets
  # Prepares markup. Renders table without any data
  class Renderer
    # Initializes renderer with tablet and params
    def initialize(tablet, params, opts = {})
      @tablet = tablet
      @params = params
      @initialize_instantly = opts.fetch(:initialize_instantly, false)
    end

    # Renders table in view_context
    def render(view_context)
      view_context.render(partial: 'tablets/tablet', locals: locals)
    end

    private

    attr_reader :params, :tablet, :initialize_instantly

    # Prepares locals for tablet partial
    def locals
      {
        id: id,
        name: tablet.name,
        columns: tablet.columns,
        options: resulting_options,
        params: params,
        initialize_instantly: initialize_instantly
      }
    end

    # Id for HTML container. Includes name and params with values
    #
    #  renderer = Tablets::Renderer.new(:posts, user_id: 1)
    #  renderer.id #=> 'posts_user_id_1'
    #
    def id
      [tablet.name, params.to_a, 'tablet'].flatten.map(&:to_s).join('_')
    end

    # Calculates resulting options.
    def resulting_options
      {}.merge(Tablets.options)
        .merge(columnDefs: columns_definitions, ajax: data_path)
        .merge(tablet.options)
    end

    # Calculates column options and applies options on top
    #
    #   ...
    #   columns do
    #     [{
    #       title: 'Title',
    #       order: 'posts.title',
    #       data: :title,
    #       options: { className: 'post-title' }
    #     }]
    #   end
    #   ...
    #
    #   renderer.columns_definitions #=> [{
    #     targets: 0,
    #     searchable: false,
    #     orderable: true,
    #     className: 'post-title'
    #   }]
    #
    def columns_definitions
      tablet.columns.map.with_index do |column, index|
        {
          targets:    index,
          searchable: column[:search].present?,
          orderable:  column[:order].present?
        }.merge!(column[:options] || {})
      end
    end

    # Returns path to data
    def data_path
      Tablets::Engine.routes.url_helpers.data_path(tablet.name)
    end
  end
end
