require 'active_support/core_ext/object/blank'

module Tablets
  module Utils
    # Config utility. Allows to write configs in declarative form.
    # And fetch values with defaults:
    #
    #   config = Config.new do
    #     some_var 32
    #     another_var 42
    #
    #     some_callback do |arg|
    #       do_something_with arg
    #     end
    #   end
    #
    #   config.get(:some_var) #=> 32
    #   config.get(:non_existent_var) #=> ArgumentError
    #   config.get(:non_existent_var, 'default') #=> 'default'
    #
    #   config.call(:some_callback, 'my_arg') { |arg| default_action arg }
    #
    class Config
      # Initializes config with block
      # Block is optional and can be applied later
      def initialize(&block)
        @hash = {}

        apply(&block) unless block.nil?
      end

      # Executes block in config context
      def apply(&block)
        instance_eval(&block)
      end

      # Checks if value or callback is defined with specified name
      def has?(name)
        @hash[name].present?
      end

      # Returns value
      # If no value defined, returns default
      # If no default raises ArgumentError
      def get(name, default = nil, &default_block)
        value = @hash[name][0] if @hash[name]

        if !value.nil?
          value
        elsif !default.nil?
          default
        elsif !default_block.nil?
          default_block.call
        else
          fail ArgumentError, "Value :#{name} is not set."
        end
      end

      # Calls callback
      # If no calbback defined, calls default
      # If no default raises ArgumentError
      def call(name, *params, &default)
        callback = @hash[name][0] if @hash[name]

        if !callback.nil?
          callback.call(*params)
        elsif !default.nil?
          default.call(*params)
        else
          fail ArgumentError, "Callback :#{name} is not registered."
        end
      end

      private

      # Gathers all calls
      def method_missing(name, *args, &block)
        @hash[name] = [*args, block]
      end
    end
  end
end
