require 'rails'
require 'active_record'

module Tablets
  module Utils
    # Resposible for building database specific search conditions
    class SearchConditionBuilder
      attr_reader :column, :query

      # Initializes builder
      def initialize(column, query)
        @column = column
        @query = query
      end

      # Builds search condition for specific db type
      def build
        sanitize "(#{prepared_column} #{like} ?)", "%#{query}%"
      end

      private

      # Sanitizes sql expression with params
      def sanitize(sql, *params)
        ActiveRecord::Base.send(:sanitize_sql_array, [sql, *params])
      end

      # Returns column prepared for using in SQL statement
      def prepared_column
        cast quote @column
      end

      # Cast column to string
      def cast(column)
        "CAST (#{column} AS #{string_type})"
      end

      # Quotes column name
      def quote(column)
        column.split('.').map do |component|
          ActiveRecord::Base.connection.quote_column_name component
        end.join('.')
      end

      # Returns database specific like operator
      def like
        case db_adapter
        when :postgresql then 'ILIKE'
        when :mysql2 then 'LIKE'
        when :sqlite3 then 'LIKE'
        end
      end

      # Returns database specific string type
      def string_type
        case db_adapter
        when :postgresql then 'VARCHAR'
        when :mysql2 then 'CHAR'
        when :sqlite3 then 'TEXT'
        end
      end

      # Retrieves rails database adapter
      def db_adapter
        @db_adapter ||=
        ActiveRecord::Base.configurations[Rails.env]['adapter'].to_sym
      end
    end
  end
end
